@@ -1,13 +1,12 @@
 use std::{env, time::Duration};
 
-use futures_util::{StreamExt, TryStreamExt};
+use futures_util::StreamExt;
 use rusoto_cloudformation::{CloudFormation, CloudFormationClient, DeleteStackInput};
 use rusoto_core::HttpClient;
 use rusoto_credential::{AutoRefreshingProvider, ChainProvider};
 
 use cloudformatious::{
-    ApplyError, ApplyEvent, ApplyInput, CloudFormatious, ResourceStatus, StackStatus,
-    TemplateSource,
+    ApplyError, ApplyInput, CloudFormatious, ResourceStatus, StackStatus, TemplateSource,
 };
 
 const NAME_PREFIX: &str = "rusoto-cloudformation-ext-testing-";
@@ -59,30 +58,26 @@ async fn create_stack_stream_ok() -> Result<(), Box<dyn std::error::Error>> {
 
     let stack_name = generated_name();
     let input = ApplyInput::new(&stack_name, TemplateSource::inline(DUMMY_TEMPLATE));
-    let events: Vec<_> = client
-        .apply(input)
-        .map_ok(|event| match event {
-            ApplyEvent::Event(event) => (
-                "event",
+    let mut apply = client.apply(input);
+
+    let events: Vec<_> = apply
+        .events()
+        .map(|event| {
+            (
                 event.logical_resource_id().to_string(),
                 event.resource_status().to_string(),
-            ),
-            ApplyEvent::Output(output) => {
-                ("output", output.stack_name, output.stack_status.to_string())
-            }
+            )
         })
-        .try_collect()
-        .await?;
+        .collect()
+        .await;
+    let output = apply.await?;
+
+    assert_eq!(output.stack_status, StackStatus::CreateComplete);
     assert_eq!(
         events,
         vec![
-            (
-                "event",
-                stack_name.clone(),
-                "CREATE_IN_PROGRESS".to_string()
-            ),
-            ("event", stack_name.clone(), "CREATE_COMPLETE".to_string()),
-            ("output", stack_name.clone(), "CREATE_COMPLETE".to_string())
+            (stack_name.clone(), "CREATE_IN_PROGRESS".to_string()),
+            (stack_name.clone(), "CREATE_COMPLETE".to_string()),
         ]
     );
 
@@ -136,18 +131,21 @@ async fn create_stack_fut_err() -> Result<(), Box<dyn std::error::Error>> {
         assert_eq!(stack_status, StackStatus::RollbackComplete);
         assert!(stack_status_reason.contains("resource(s) failed to create: [Vpc]"));
         assert_eq!(
-            resource_events.get(0).map(|(status, details)| {
-                (
-                    details.logical_resource_id(),
-                    *status,
-                    details.resource_status_reason(),
-                )
-            }),
-            Some((
+            resource_events
+                .iter()
+                .map(|(status, details)| {
+                    (
+                        details.logical_resource_id(),
+                        *status,
+                        details.resource_status_reason(),
+                    )
+                })
+                .collect::<Vec<_>>(),
+            vec![(
                 "Vpc",
                 ResourceStatus::CreateFailed,
                 Some("Property CidrBlock cannot be empty.")
-            ))
+            )]
         );
     } else {
         return Err(error.into());
@@ -170,71 +168,59 @@ async fn create_stack_stream_err() -> Result<(), Box<dyn std::error::Error>> {
 
     let stack_name = generated_name();
     let input = ApplyInput::new(&stack_name, TemplateSource::inline(FAILING_TEMPLATE));
-    let events: Vec<_> = client
-        .apply(input)
-        .map(|event| match event {
-            Ok(ApplyEvent::Event(event)) => Ok((
-                "event",
+    let mut apply = client.apply(input);
+
+    let events: Vec<_> = apply
+        .events()
+        .map(|event| {
+            (
                 event.logical_resource_id().to_string(),
                 event.resource_status().to_string(),
-            )),
-            Ok(ApplyEvent::Output(output)) => {
-                panic!("expected err but stack {} succeeded", output.stack_id)
-            }
-            Err(ApplyError::Failure {
-                stack_status,
-                stack_status_reason,
-                resource_events,
-                ..
-            }) => {
-                assert!(stack_status_reason.contains("resource(s) failed to create: [Vpc]"));
-                assert_eq!(
-                    resource_events
-                        .iter()
-                        .map(|(status, details)| {
-                            (
-                                details.logical_resource_id(),
-                                *status,
-                                details.resource_status_reason(),
-                            )
-                        })
-                        .collect::<Vec<_>>(),
-                    vec![(
-                        "Vpc",
-                        ResourceStatus::CreateFailed,
-                        Some("Property CidrBlock cannot be empty.")
-                    )]
-                );
-                Ok(("output", stack_name.clone(), stack_status.to_string()))
-            }
-            Err(error) => Err(error),
+            )
         })
-        .try_collect()
-        .await?;
+        .collect()
+        .await;
+    let error = apply.await.unwrap_err();
 
     assert_eq!(
         events,
         vec![
-            (
-                "event",
-                stack_name.clone(),
-                "CREATE_IN_PROGRESS".to_string()
-            ),
-            ("event", "Vpc".to_string(), "CREATE_FAILED".to_string()),
-            (
-                "event",
-                stack_name.clone(),
-                "ROLLBACK_IN_PROGRESS".to_string()
-            ),
-            ("event", "Vpc".to_string(), "DELETE_COMPLETE".to_string()),
-            ("event", stack_name.clone(), "ROLLBACK_COMPLETE".to_string()),
-            (
-                "output",
-                stack_name.clone(),
-                "ROLLBACK_COMPLETE".to_string()
-            )
+            (stack_name.clone(), "CREATE_IN_PROGRESS".to_string()),
+            ("Vpc".to_string(), "CREATE_FAILED".to_string()),
+            (stack_name.clone(), "ROLLBACK_IN_PROGRESS".to_string()),
+            ("Vpc".to_string(), "DELETE_COMPLETE".to_string()),
+            (stack_name.clone(), "ROLLBACK_COMPLETE".to_string()),
         ]
     );
+    if let ApplyError::Failure {
+        stack_status,
+        stack_status_reason,
+        resource_events,
+        ..
+    } = error
+    {
+        assert_eq!(stack_status, StackStatus::RollbackComplete);
+        assert!(stack_status_reason.contains("resource(s) failed to create: [Vpc]"));
+        assert_eq!(
+            resource_events
+                .iter()
+                .map(|(status, details)| {
+                    (
+                        details.logical_resource_id(),
+                        *status,
+                        details.resource_status_reason(),
+                    )
+                })
+                .collect::<Vec<_>>(),
+            vec![(
+                "Vpc",
+                ResourceStatus::CreateFailed,
+                Some("Property CidrBlock cannot be empty.")
+            )]
+        );
+    } else {
+        return Err(error.into());
+    }
 
     // Clean-up
     client
@@ -283,18 +269,21 @@ async fn update_stack_fut_err() -> Result<(), Box<dyn std::error::Error>> {
         assert_eq!(stack_status, StackStatus::UpdateRollbackComplete);
         assert!(stack_status_reason.contains("resource(s) failed to create: [Vpc]"));
         assert_eq!(
-            resource_events.get(0).map(|(status, details)| {
-                (
-                    details.logical_resource_id(),
-                    *status,
-                    details.resource_status_reason(),
-                )
-            }),
-            Some((
+            resource_events
+                .iter()
+                .map(|(status, details)| {
+                    (
+                        details.logical_resource_id(),
+                        *status,
+                        details.resource_status_reason(),
+                    )
+                })
+                .collect::<Vec<_>>(),
+            vec![(
                 "Vpc",
                 ResourceStatus::CreateFailed,
                 Some("Property CidrBlock cannot be empty.")
-            ))
+            )]
         );
     } else {
         return Err(error.into());
